; servopulse_512.a51: A pulse-width program, using two timers  5/3/01, tidied 12/04: short first-pass delay, longer thereafter
;  value entered on one key at keypad, range 4 through F, sets pulse width appropriately
;  to control an RC-hobbyist "servo motor" (roughly 0.2ms to 1.5 ms pulse)

; May 12: 1) narrow the output pulses by using 13-bit timer0 mode rather than 16;
;  2) increase sensitivity to single-key entry by shifting left 4 places...
;  3) then back off slightly by multiplying by DCh (keep 85% of shifted value)
; Range of motion: hits limits at values 4 and F; 5 to E are just short of the limits

;  (4/08) re-assign output pin to P1.0, to avoid conflict with serial port

    $NOSYMBOLS        ; keeps listing short

; default Ride7 path, Mar. 16
    $INCLUDE (C:\Program Files\Raisonance\Ride\inc\51\Dallas\REG320.INC) 
    $INCLUDE (C:\Program Files\Raisonance\Ride\inc\51\Dallas\vectors320.INC) 
	
	 HISTART SET 0FFh	; set 1st-pass start count close to the overflow val, for debugging ease (1st pass only)
	 LOSTART SET 0E0h	;  ...and this is the low byte
	 
	 HITIME SET 080h	; high half of reload value, for general run.  Puts rep rate at about 30Hz

	 LOTIME SET 00h	; ...low half
	 MULTIPLIER EQU 0DCh   ; multiplier, to keep 85% of shifted key value
	
	 STACKBOT EQU 80h	; scratch RAM, OK place to put stack

	 GLOBALINTEN EQU IE.7	; these are private names, meant to be readable
	 TIM2INTEN EQU IE.5

	 CLRDCEN SET 0F7h		; mask to clear DCEN bit, for up count
	 DIV4 SET 20h		; single bit to set hi for prescale to "/4" in CKCON
	 ENBLTMR2 EQU T2CON.2	; a handy shorthand for this bit
	 

	ORG 0H

	JMP START

	ORG 1D0h

START: MOV SP, #STACKBOT-1
	 ACALL    INITS         ;GO INITIALIZE


	MOV RCAP2H, #HITIME		; Init timer registers 
	MOV RCAP2L, #LOTIME		;  timer counts UP from initial value
	SETB ENBLTMR2		; ...and enable timer 2

	CLR P1.0				; clear output pulse, intially
LOITER:	JNB 20h,LOITER	; hang here till flag bit goes up
	CLR 20h				; clear the flag, when it comes up
	ACALL MAKEPULSE		; now send a pulse
	SJMP LOITER				; AWAIT INTERRUPTS, with pulse now low again

;-------MAKEPULSE: USES TIMER 0, to make pulse, whose width is determined by key value
MAKEPULSE:	MOVX A,@DPTR			; read key value
	JZ MAKEPULSE				; no pulse, if key value is zero
	ANL A, #0Fh				; mask out all but most recent key
	RL A						; SHRINK BY FACTOR OF 4 (4/08) (boost value; we're counting UP)
	RL A
	RL A
	RL A                 ; we've boosted effect 16X
	MOV B, #MULTIPLIER   ; now back off: get partial mutliplier value
	MUL AB
	MOV A, B             ; use MSD of multiplied result

	MOV TH0,A				; ...load it as delay value (MSD)
	SETB TR0				; enable timer 0
	
GOHI:	SETB P1.0				; send output high
HANGHIGH: JNB TF0,HANGHIGH	; stay till overflow flag is asserted
	CLR TF0
	CLR P1.0					; terminate pulse
	CLR TR0				; disable counter 0
	RET
	
;------------------------------
; ISR: SET FLAG THAT WILL INVOKE SECOND TIMER
	ORG TIMER2VECTOR

ISR:	SETB 20h	; set a flag bit for MAIN
	CLR TF2		; Clear timer-2 overflow flag, as defined
					;  among Dallas registers listed by Raisonance
	RETI	


; SUBROUTINE*******************************************
	ORG 210h
; SOME INITIALIZATIONS
INITS:	MOV DPTR, #8000h	; INIT DATA POINTER

	ANL T2MOD,#CLRDCEN	; clear DCEN bit: sets counter to UP
								;  (reiterates default)
;	ORL CKCON,#DIV4		; set timer-prescaler to faster--div by 4
;	SET TIMER HI, TO SHORTEN FIRST-PASS
	MOV TH2, #HISTART
	MOV TL2, #LOSTART
	
; here come TIMER0 inits:
; comment out next line, after pulses seem much too long 4/08	RESTORED
	ORL CKCON,#08h			; force bit 3 of CKCON high: set div-by at 4, for timer 0--3X faster than default
	ANL TMOD, #0F3h		; force bits 3 & 2 low: gate low, to enable timer 0
								; select internal clock as source for timer 0
; run it as 13-bit timer--comment out next line
;	ORL TMOD, #01h			; ...and make it a 16-bit timer
	
; ----NOW ENABLE INTERRUPT, timer 2

	SETB TIM2INTEN		; Global timer-2 interrupt enable
	SETB GLOBALINTEN	; Global int enable
	RET
	
	END
